==> visitors-8.x-2.17.tar.gz: visitors/src/Service/TrackerService.php <==
<?php

namespace Drupal\visitors\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Utility\Error;
use Drupal\visitors\Event\VisitLogEvent;
use Drupal\visitors\VisitorsCookieInterface;
use Drupal\visitors\VisitorsCounterInterface;
use Drupal\visitors\VisitorsDeviceInterface;
use Drupal\visitors\VisitorsTrackerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Tracker for web analytics.
 */
class TrackerService implements VisitorsTrackerInterface {

  /**
   * The request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The date service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The counter service.
   *
   * @var \Drupal\visitors\VisitorsCounterInterface
   */
  protected $counter;

  /**
   * The cookie service.
   *
   * @var \Drupal\visitors\VisitorsCookieInterface
   */
  protected $cookie;

  /**
   * The device service.
   *
   * @var \Drupal\visitors\VisitorsDeviceInterface
   */
  protected $device;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Tracks visits and actions.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Component\Datetime\TimeInterface $time_service
   *   The date service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\visitors\VisitorsCounterInterface $counter
   *   The counter service.
   * @param \Drupal\visitors\VisitorsCookieInterface $cookie
   *   The cookie service.
   * @param \Drupal\visitors\VisitorsDeviceInterface $device
   *   The device service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(
    RequestStack $request_stack,
    TimeInterface $time_service,
    ModuleHandlerInterface $module_handler,
    Connection $database,
    EventDispatcherInterface $event_dispatcher,
    VisitorsCounterInterface $counter,
    VisitorsCookieInterface $cookie,
    VisitorsDeviceInterface $device,
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger,
  ) {

    $this->request = $request_stack->getCurrentRequest();
    $this->time = $time_service;
    $this->moduleHandler = $module_handler;
    $this->database = $database;
    $this->eventDispatcher = $event_dispatcher;
    $this->counter = $counter;
    $this->cookie = $cookie;
    $this->device = $device;
    $this->settings = $config_factory->get('visitors.config');
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function log(array $agent): void {
    $path = '';
    $route = '';
    $viewed = NULL;
    $bot_retention_log = $this->settings->get('bot_retention_log') ?? 0;
    $discard_bot = $bot_retention_log == -1;
    $ip = $this->request->getClientIp();
    $custom = json_decode($agent['_cvar'] ?? '{}');
    foreach ($custom as $c) {
      if ($c[0] == 'path') {
        $path = $c[1];
      }
      if ($c[0] == 'route') {
        $route = $c[1];
      }
      if ($c[0] == 'viewed') {
        $viewed = $c[1];
      }
    }

    $visitor_localtime = $this->getLocaltime($agent);

    $visitor_id = $this->getVisitorId($agent);
    $fields = [
      'visitor_id'          => $visitor_id,
      'visitor_localtime'   => $visitor_localtime,
      'visitors_uid'        => $agent['uid'] ?? 0,
      'visitors_ip'         => $ip,
      'visitors_date_time'  => $this->time->getRequestTime(),
      'visitors_url'        => $agent['url'] ?? '',
      'visitors_referer'    => $agent['urlref'] ?? '',
      'visitors_path'       => $path,
      'visitors_title'      => $agent['action_name'] ?? '',
      'visitors_user_agent' => $this->getUserAgent(),
      'route'               => $route,
      'config_resolution'   => $agent['res'] ?? NULL,
      'config_pdf'          => $agent['pdf'] ?? NULL,
      'config_flash'        => $agent['fla'] ?? NULL,
      'config_java'         => $agent['java'] ?? NULL,
      'config_quicktime'    => $agent['qt'] ?? NULL,
      'config_realplayer'   => $agent['realp'] ?? NULL,
      'config_windowsmedia' => $agent['wma'] ?? NULL,
      'config_silverlight'  => $agent['ag'] ?? NULL,
      'config_cookie'       => $agent['cookie'] ?? NULL,
    ];

    $this->doDeviceDetect($fields);
    if ($discard_bot && $fields['bot']) {
      return;
    }

    $event = new VisitLogEvent($fields);
    $this->eventDispatcher->dispatch($event);
    $fields = $event->getFields();

    $log_performance = $this->settings->get('performance') ?? 0;

    try {
      $id = $this->database->insert('visitors')
        ->fields($fields)
        ->execute();

      $performance = $log_performance && $id ? TRUE : FALSE;
      if ($performance) {
        $this->logPerformance($id, $agent);
      }

      if (!is_null($viewed)) {
        [$type, $id] = explode(':', $viewed);
        $this->counter->recordView($type, $id);
      }
    }
    catch (\Exception $e) {
      if (version_compare(\Drupal::VERSION, '10.1', '>=')) {
        Error::logException($this->logger, $e);
      }
      else {
        // @phpstan-ignore-next-line
        watchdog_exception('visitors', $e);
      }
    }
  }

  /**
   * Write performance metrics to the database.
   *
   * @param int $visitors_id
   *   The visitors id.
   * @param string[] $agent
   *   The agent array.
   */
  protected function logPerformance(int $visitors_id, array $agent): void {
    $performance = [
      'network' => $agent['pf_net'] ?? 0,
      'server' => $agent['pf_srv'] ?? 0,
      'transfer' => $agent['pf_tfr'] ?? 0,
      'dom_processing' => $agent['pf_dm1'] ?? 0,
      'dom_complete' => $agent['pf_dm2'] ?? 0,
      'on_load' => $agent['pf_onl'] ?? 0,
      'visitors_id' => $visitors_id,
    ];
    $performance['total'] = $performance['network']
      + $performance['server']
      + $performance['transfer']
      + $performance['dom_processing']
      + $performance['dom_complete']
      + $performance['on_load'];

    $this->database->insert('visitors_performance')
      ->fields($performance)
      ->execute();
  }

  /**
   * Get visitor user agent.
   *
   * @return string
   *   string user agent, or empty string if user agent does not exist
   */
  protected function getUserAgent() {
    return $_SERVER['HTTP_USER_AGENT'] ?? '';
  }

  /**
   * Get local time.
   *
   * @param array $agent
   *   The hours.
   *
   * @return int|null
   *   The time.
   */
  protected function getLocaltime($agent) {

    $hours = $agent['h'] ?? NULL;
    $minutes = $agent['m'] ?? NULL;
    $seconds = $agent['s'] ?? NULL;

    $has_null = is_null($hours) || is_null($minutes) || is_null($seconds);
    if ($has_null) {
      return NULL;
    }

    $time = $hours * 3600 + $minutes * 60 + $seconds;

    return $time;
  }

  /**
   * Get visitor id.
   *
   * @param string[] $agent
   *   The agent array.
   *
   * @return string
   *   The visitor id.
   */
  protected function getVisitorId(array $agent) {
    return $agent['_id'] ?? $this->cookie->getId();
  }

  /**
   * Detects the device.
   *
   * @param string[] $fields
   *   The fields array.
   */
  protected function doDeviceDetect(array &$fields) {
    $dd = $this->device->getDeviceDetector($this->getUserAgent(), $_SERVER);

    $fields['config_browser_engine'] = $dd->getClient('engine');
    $fields['config_browser_name'] = $dd->getClient('name');
    $fields['config_browser_version'] = $dd->getClient('version');
    $fields['config_client_type'] = $dd->getClient('type');
    $fields['config_device_brand'] = $dd->getBrandName();
    $fields['config_device_model'] = $dd->getModel();
    $fields['config_device_type'] = $dd->getDeviceName();
    $fields['config_os'] = $dd->getOs('short_name');
    $fields['config_os_version'] = $dd->getOs('version');
    $fields['bot'] = (int) $dd->isBot();

  }

}

==> visitors-8.x-2.18.tar.gz: visitors/src/Service/TrackerService.php <==
<?php

namespace Drupal\visitors\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Utility\Error;
use Drupal\visitors\Event\VisitLogEvent;
use Drupal\visitors\VisitorsCookieInterface;
use Drupal\visitors\VisitorsCounterInterface;
use Drupal\visitors\VisitorsDeviceInterface;
use Drupal\visitors\VisitorsTrackerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Tracker for web analytics.
 */
class TrackerService implements VisitorsTrackerInterface {

  /**
   * The request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The date service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The counter service.
   *
   * @var \Drupal\visitors\VisitorsCounterInterface
   */
  protected $counter;

  /**
   * The cookie service.
   *
   * @var \Drupal\visitors\VisitorsCookieInterface
   */
  protected $cookie;

  /**
   * The device service.
   *
   * @var \Drupal\visitors\VisitorsDeviceInterface
   */
  protected $device;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Tracks visits and actions.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Component\Datetime\TimeInterface $time_service
   *   The date service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\visitors\VisitorsCounterInterface $counter
   *   The counter service.
   * @param \Drupal\visitors\VisitorsCookieInterface $cookie
   *   The cookie service.
   * @param \Drupal\visitors\VisitorsDeviceInterface $device
   *   The device service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(RequestStack $request_stack, TimeInterface $time_service, ModuleHandlerInterface $module_handler, Connection $database, EventDispatcherInterface $event_dispatcher, VisitorsCounterInterface $counter, VisitorsCookieInterface $cookie, VisitorsDeviceInterface $device, ConfigFactoryInterface $config_factory, LoggerInterface $logger) {

    $this->request = $request_stack->getCurrentRequest();
    $this->time = $time_service;
    $this->moduleHandler = $module_handler;
    $this->database = $database;
    $this->eventDispatcher = $event_dispatcher;
    $this->counter = $counter;
    $this->cookie = $cookie;
    $this->device = $device;
    $this->settings = $config_factory->get('visitors.config');
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function log(array $agent): void {
    $path = '';
    $route = '';
    $viewed = NULL;
    $bot_retention_log = $this->settings->get('bot_retention_log') ?? 0;
    $discard_bot = $bot_retention_log == -1;
    $ip = $this->request->getClientIp();
    $custom = json_decode($agent['_cvar'] ?? '{}');
    foreach ($custom as $c) {
      if ($c[0] == 'path') {
        $path = $c[1];
      }
      if ($c[0] == 'route') {
        $route = $c[1];
      }
      if ($c[0] == 'viewed') {
        $viewed = $c[1];
      }
    }

    $visitor_localtime = $this->getLocaltime($agent);

    $visitor_id = $this->getVisitorId($agent);
    $fields = [
      'visitor_id'          => $visitor_id,
      'visitor_localtime'   => $visitor_localtime,
      'visitors_uid'        => $agent['uid'] ?? 0,
      'visitors_ip'         => $ip,
      'visitors_date_time'  => $this->time->getRequestTime(),
      'visitors_url'        => $agent['url'] ?? '',
      'visitors_referer'    => $agent['urlref'] ?? '',
      'visitors_path'       => $path,
      'visitors_title'      => $agent['action_name'] ?? '',
      'visitors_user_agent' => $this->getUserAgent(),
      'route'               => $route,
      'config_resolution'   => $agent['res'] ?? NULL,
      'config_pdf'          => $agent['pdf'] ?? NULL,
      'config_flash'        => $agent['fla'] ?? NULL,
      'config_java'         => $agent['java'] ?? NULL,
      'config_quicktime'    => $agent['qt'] ?? NULL,
      'config_realplayer'   => $agent['realp'] ?? NULL,
      'config_windowsmedia' => $agent['wma'] ?? NULL,
      'config_silverlight'  => $agent['ag'] ?? NULL,
      'config_cookie'       => $agent['cookie'] ?? NULL,
    ];

    $this->doDeviceDetect($fields);
    if ($discard_bot && $fields['bot']) {
      return;
    }

    $event = new VisitLogEvent($fields);
    $this->eventDispatcher->dispatch($event);
    $fields = $event->getFields();

    $log_performance = $this->settings->get('performance') ?? 0;

    try {
      $id = $this->database->insert('visitors')
        ->fields($fields)
        ->execute();

      $performance = $log_performance && $id ? TRUE : FALSE;
      if ($performance) {
        $this->logPerformance($id, $agent);
      }

      if (!is_null($viewed)) {
        [$type, $id] = explode(':', $viewed);
        $this->counter->recordView($type, $id);
      }
    }
    catch (\Exception $e) {
      if (version_compare(\Drupal::VERSION, '10.1', '>=')) {
        Error::logException($this->logger, $e);
      }
      else {
        // @phpstan-ignore-next-line
        watchdog_exception('visitors', $e);
      }
    }
  }

  /**
   * Write performance metrics to the database.
   *
   * @param int $visitors_id
   *   The visitors id.
   * @param string[] $agent
   *   The agent array.
   */
  protected function logPerformance(int $visitors_id, array $agent): void {
    $performance = [
      'network' => $agent['pf_net'] ?? 0,
      'server' => $agent['pf_srv'] ?? 0,
      'transfer' => $agent['pf_tfr'] ?? 0,
      'dom_processing' => $agent['pf_dm1'] ?? 0,
      'dom_complete' => $agent['pf_dm2'] ?? 0,
      'on_load' => $agent['pf_onl'] ?? 0,
      'visitors_id' => $visitors_id,
    ];
    $performance['total'] = $performance['network']
      + $performance['server']
      + $performance['transfer']
      + $performance['dom_processing']
      + $performance['dom_complete']
      + $performance['on_load'];

    $this->database->insert('visitors_performance')
      ->fields($performance)
      ->execute();
  }

  /**
   * Get visitor user agent.
   *
   * @return string
   *   string user agent, or empty string if user agent does not exist
   */
  protected function getUserAgent() {
    return $_SERVER['HTTP_USER_AGENT'] ?? '';
  }

  /**
   * Get local time.
   *
   * @param array $agent
   *   The hours.
   *
   * @return int|null
   *   The time.
   */
  protected function getLocaltime($agent) {

    $hours = $agent['h'] ?? NULL;
    $minutes = $agent['m'] ?? NULL;
    $seconds = $agent['s'] ?? NULL;

    $has_null = is_null($hours) || is_null($minutes) || is_null($seconds);
    if ($has_null) {
      return NULL;
    }

    $time = $hours * 3600 + $minutes * 60 + $seconds;

    return $time;
  }

  /**
   * Get visitor id.
   *
   * @param string[] $agent
   *   The agent array.
   *
   * @return string
   *   The visitor id.
   */
  protected function getVisitorId(array $agent) {
    return $agent['_id'] ?? $this->cookie->getId();
  }

  /**
   * Detects the device.
   *
   * @param string[] $fields
   *   The fields array.
   */
  protected function doDeviceDetect(array &$fields) {
    $dd = $this->device->getDeviceDetector($this->getUserAgent(), $_SERVER);

    $fields['config_browser_engine'] = $dd->getClient('engine');
    $fields['config_browser_name'] = $dd->getClient('short_name');
    $fields['config_browser_version'] = $dd->getClient('version');
    $fields['config_client_type'] = $dd->getClient('type');
    $fields['config_device_brand'] = $dd->getBrandName();
    $fields['config_device_model'] = $dd->getModel();
    $fields['config_device_type'] = $dd->getDeviceName();
    $fields['config_os'] = $dd->getOs('short_name');
    $fields['config_os_version'] = $dd->getOs('version');
    $fields['bot'] = (int) $dd->isBot();

  }

}
